/*
 * MIT License
 * 
 * Copyright (c) 2020 Alessandro Mauri
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <poll.h>
#include <sys/inotify.h>
#include "util.h"
#include "disk.h"

#define DEV_BLOCK_ROOT "/dev/block/"

// FIXME: this is all very inefficient code, everything gets called multiple
// times and it is not at all minimal
int main (int argc, char *argv[])
{
	struct pollfd watchpoll = {0, POLLIN, 0};
	const struct inotify_event *event;
	char buf[4096], strbuf[256];
	char *ptr;
	int opt, len, fd;

	while ((opt = getopt(argc, argv, "h")) != -1) {
		switch (opt) {
		case 'h':
			usage();
			exit(EXIT_SUCCESS);
			break;
		default:
			puts(red("Option not recognized\n"));
			usage();
			exit(EXIT_FAILURE);
			break;
		}
	}

	if ((fd = inotify_init()) < 0)
		die("error initializing inotify watch list");
	watchpoll.fd = fd;
	if (inotify_add_watch(fd, DEV_BLOCK_ROOT, IN_CREATE | IN_DELETE) < 0)
		die ("error adding " DEV_BLOCK_ROOT " to the watch list");

	for (;poll(&watchpoll, 1, -1);) {
		if (watchpoll.revents != POLLIN)
			continue;

		len = read(watchpoll.fd, buf, sizeof(buf));
		if (len < 0) {
			printf(red("read error: %s\n"), strerror(errno));
			break;
		} else if (!len) {
			puts("No data to read\n");
			break;
		}

		for (ptr = buf; ptr < buf + len;
		ptr += sizeof(struct inotify_event) + event->len) {
			event = (const struct inotify_event *) ptr;
			if(!event->len)
				continue;
			strcpy(strbuf, DEV_BLOCK_ROOT);
			strcat(strbuf, event->name);

			printf(yellow("%s: %s - %s"),
			event->mask & IN_CREATE ? "Created" : "Removed",
			get_path(strbuf),
			path_is_disk(get_path(strbuf)) > 0 ? "disk" : "part");
			printf(yellow("\n"));

		}
	}
	die("polling interrupted");

	close(watchpoll.fd);
	return 0;
}


