/*
 * MIT License
 * 
 * Copyright (c) 2020 Alessandro Mauri
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#ifndef _DISK_H
#define _DISK_H

/* Block info data type */
// TODO: maybe split this into three structs, one generic list, one just disk
// and one just for partitions, idk
struct partition_list {
		const char *name;
		struct partition_list *next;
};

struct disk_list {
	const char *name;
	const char *block_path;
	struct partition_list *parts;
	struct disk_list *next;	
};

int disk_add_disk (struct disk_list *, const char *);
int path_is_disk (const char *);

struct disk_list * disk_get_by_name (struct disk_list *, const char *);
void disk_rm_by_name (struct disk_list **, const char *);
void disk_list_free (struct disk_list **);
void disk_list_add (struct disk_list *, struct disk_list *);

struct partition_list * disk_part_get_by_name (struct disk_list *, const char *);
void partition_rm_by_name (struct partition_list *, const char *);
void partition_list_free (struct partition_list *);
void partition_list_add (struct partition_list *, struct partition_list *);

#endif
